@@ -125,6 +125,32 @@ cogl_clip_push_from_path (void)
   ctx->current_path = cogl2_path_new ();
 }
 
+void
+cogl_clip_push_primitive (CoglPrimitive *primitive,
+                          float bounds_x1,
+                          float bounds_y1,
+                          float bounds_x2,
+                          float bounds_y2)
+{
+  CoglFramebuffer *framebuffer;
+  CoglClipState *clip_state;
+  CoglMatrix modelview_matrix;
+
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  framebuffer = cogl_get_draw_framebuffer ();
+  clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
+
+  cogl_get_modelview_matrix (&modelview_matrix);
+
+  clip_state->stacks->data =
+    _cogl_clip_stack_push_primitive (clip_state->stacks->data,
+                                     primitive,
+                                     bounds_x1, bounds_y1,
+                                     bounds_x2, bounds_y2,
+                                     &modelview_matrix);
+}
+
 static void
 _cogl_clip_pop_real (CoglClipState *clip_state)
 {
@@ -159,16 +185,12 @@ _cogl_clip_state_flush (CoglClipState *clip_state,
 void
 cogl_clip_ensure (void)
 {
-  CoglFramebuffer *framebuffer = cogl_get_draw_framebuffer ();
-  CoglClipState *clip_state;
-
-  clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
-  /* Flushing the clip state doesn't cause the journal to be
-     flushed. This function may be being called by an external
-     application however so it makes sense to flush the journal
-     here */
-  _cogl_framebuffer_flush_journal (framebuffer);
-  _cogl_clip_state_flush (clip_state, framebuffer);
+  /* Do nothing.
+   *
+   * This API shouldn't be used by anyone and the documented semantics
+   * are basically vague enough that we can get away with doing
+   * nothing here.
+   */
 }
 
 static void
@@ -196,7 +218,7 @@ _cogl_clip_stack_restore_real (CoglClipState *clip_state)
 {
   CoglHandle stack;
 
-  g_return_if_fail (clip_state->stacks != NULL);
+  _COGL_RETURN_IF_FAIL (clip_state->stacks != NULL);
 
   stack = clip_state->stacks->data;
 
