@@ -109,6 +109,8 @@ cogl_display_new (CoglRenderer *renderer,
   display->onscreen_template = onscreen_template;
   if (onscreen_template)
     cogl_object_ref (onscreen_template);
+  else
+    display->onscreen_template = cogl_onscreen_template_new (NULL);
 
   display->setup = FALSE;
 
@@ -119,6 +121,12 @@ cogl_display_new (CoglRenderer *renderer,
   return _cogl_display_object_new (display);
 }
 
+CoglRenderer *
+cogl_display_get_renderer (CoglDisplay *display)
+{
+  return display->renderer;
+}
+
 gboolean
 cogl_display_setup (CoglDisplay *display,
                     GError **error)
@@ -142,7 +150,7 @@ void
 cogl_gdl_display_set_plane (CoglDisplay *display,
                             gdl_plane_id_t plane)
 {
-  g_return_if_fail (display->setup == FALSE);
+  _COGL_RETURN_IF_FAIL (display->setup == FALSE);
 
   display->gdl_plane = plane;
 }
@@ -153,7 +161,7 @@ void
 cogl_wayland_display_set_compositor_display (CoglDisplay *display,
                                              struct wl_display *wayland_display)
 {
-  g_return_if_fail (display->setup == FALSE);
+  _COGL_RETURN_IF_FAIL (display->setup == FALSE);
 
   display->wayland_compositor_display = wayland_display;
 }
