@@ -106,6 +106,8 @@ COGL_EXT_FUNCTION (void, glFlush,
                    (void))
 COGL_EXT_FUNCTION (void, glFrontFace,
                    (GLenum mode))
+COGL_EXT_FUNCTION (void, glCullFace,
+                   (GLenum mode))
 COGL_EXT_FUNCTION (void, glGenTextures,
                    (GLsizei n, GLuint* textures))
 COGL_EXT_FUNCTION (GLenum, glGetError,
@@ -331,18 +333,6 @@ COGL_EXT_FUNCTION (void, glBlitFramebuffer,
                     GLenum                filter))
 COGL_EXT_END ()
 
-COGL_EXT_BEGIN (offscreen_multisample, 255, 255,
-                0, /* not in either GLES */
-                "EXT\0",
-                "framebuffer_multisample\0")
-COGL_EXT_FUNCTION (void, glRenderbufferStorageMultisample,
-                   (GLenum                target,
-                    GLsizei               samples,
-                    GLenum                internalformat,
-                    GLsizei               width,
-                    GLsizei               height))
-COGL_EXT_END ()
-
 /* ARB_fragment_program */
 COGL_EXT_BEGIN (arbfp, 255, 255,
                 0, /* not in either GLES */
@@ -381,7 +371,7 @@ COGL_EXT_FUNCTION (GLuint, glCreateShader,
 COGL_EXT_FUNCTION (void, glShaderSource,
                    (GLuint                shader,
                     GLsizei               count,
-                    const GLchar        **string,
+                    const char          **string,
                     const GLint          *length))
 COGL_EXT_FUNCTION (void, glCompileShader,
                    (GLuint                shader))
@@ -396,14 +386,14 @@ COGL_EXT_FUNCTION (void, glUseProgram,
                    (GLuint                program))
 COGL_EXT_FUNCTION (GLint, glGetUniformLocation,
                    (GLuint                program,
-                    const GLchar         *name))
+                    const char           *name))
 COGL_EXT_FUNCTION (void, glDeleteProgram,
                    (GLuint                program))
 COGL_EXT_FUNCTION (void, glGetShaderInfoLog,
                    (GLuint                shader,
                     GLsizei               maxLength,
                     GLsizei              *length,
-                    GLchar               *infoLog))
+                    char                 *infoLog))
 COGL_EXT_FUNCTION (void, glGetShaderiv,
                    (GLuint                shader,
                     GLenum                pname,
@@ -514,13 +504,13 @@ COGL_EXT_FUNCTION (void, glGetProgramInfoLog,
                    (GLuint                program,
                     GLsizei               bufSize,
                     GLsizei              *length,
-                    GLchar               *infoLog))
+                    char                 *infoLog))
 
 COGL_EXT_FUNCTION (void, glVertexAttrib4f,
                    (GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w))
 
 COGL_EXT_FUNCTION (GLint, glGetAttribLocation,
-                   (GLuint program, const GLchar *name))
+                   (GLuint program, const char *name))
 
 COGL_EXT_END ()
 
@@ -647,3 +637,32 @@ COGL_EXT_FUNCTION (void, glEGLImageTargetRenderbufferStorage,
                    (GLenum           target,
                     GLeglImageOES    image))
 COGL_EXT_END ()
+
+COGL_EXT_BEGIN (framebuffer_discard, 255, 255,
+                0, /* not in either GLES */
+                "EXT\0",
+                "framebuffer_discard\0")
+COGL_EXT_FUNCTION (void, glDiscardFramebuffer,
+                   (GLenum           target,
+                    GLsizei          numAttachments,
+                    const GLenum    *attachments))
+COGL_EXT_END ()
+
+COGL_EXT_BEGIN (IMG_multisampled_render_to_texture, 255, 255,
+                0, /* not in either GLES */
+                "\0",
+                "IMG_multisampled_render_to_texture\0")
+COGL_EXT_FUNCTION (void, glRenderbufferStorageMultisampleIMG,
+                   (GLenum           target,
+                    GLsizei          samples,
+                    GLenum           internal_format,
+                    GLsizei          width,
+                    GLsizei          height))
+COGL_EXT_FUNCTION (void, glFramebufferTexture2DMultisampleIMG,
+                   (GLenum           target,
+                    GLenum           attachment,
+                    GLenum           textarget,
+                    GLuint           texture,
+                    GLint            level,
+                    GLsizei          samples))
+COGL_EXT_END ()
