@@ -1118,10 +1118,12 @@ create_attribute_buffer (CoglJournal *journal,
 {
   CoglAttributeBuffer *vbo;
 
+  _COGL_GET_CONTEXT (ctx, NULL);
+
   /* If CoglBuffers are being emulated with malloc then there's not
      really any point in using the pool so we'll just allocate the
      buffer directly */
-  if (!cogl_features_available (COGL_FEATURE_VBOS))
+  if (!(ctx->private_feature_flags & COGL_PRIVATE_FEATURE_VBOS))
     return cogl_attribute_buffer_new (n_bytes, NULL);
 
   vbo = journal->vbo_pool[journal->next_vbo_in_pool];
@@ -1449,7 +1451,7 @@ static gboolean
 add_framebuffer_deps_cb (CoglPipelineLayer *layer, void *user_data)
 {
   CoglFramebuffer *framebuffer = user_data;
-  CoglHandle texture = _cogl_pipeline_layer_get_texture_real (layer);
+  CoglTexture *texture = _cogl_pipeline_layer_get_texture_real (layer);
   const GList *l;
 
   if (!texture)
@@ -1466,7 +1468,7 @@ _cogl_journal_log_quad (CoglJournal  *journal,
                         const float  *position,
                         CoglPipeline *pipeline,
                         int           n_layers,
-                        CoglHandle    layer0_override_texture,
+                        CoglTexture  *layer0_override_texture,
                         const float  *tex_coords,
                         unsigned int  tex_coords_len)
 {
