@@ -18,8 +18,7 @@
 
 typedef struct _TestState
 {
-  int width;
-  int height;
+  int dummy;
 } TestState;
 
 typedef struct
@@ -33,28 +32,6 @@ typedef struct
   float                 range_far;
 } TestDepthState;
 
-static void
-check_pixel (GLubyte *pixel, guint32 color)
-{
-  guint8 r = MASK_RED (color);
-  guint8 g = MASK_GREEN (color);
-  guint8 b = MASK_BLUE (color);
-  guint8 a = MASK_ALPHA (color);
-
-  if (g_test_verbose ())
-    g_print ("  expected = %x, %x, %x, %x\n",
-             r, g, b, a);
-  /* FIXME - allow for hardware in-precision */
-  g_assert_cmpint (pixel[RED], ==, r);
-  g_assert_cmpint (pixel[GREEN], ==, g);
-  g_assert_cmpint (pixel[BLUE], ==, b);
-
-  /* FIXME
-   * We ignore the alpha, since we don't know if our render target is
-   * RGB or RGBA */
-  /* g_assert (pixel[ALPHA] == a); */
-}
-
 static gboolean
 draw_rectangle (TestState *state,
                 int x,
@@ -109,9 +86,6 @@ test_depth (TestState *state,
             TestDepthState *rect2_state,
             guint32 expected_result)
 {
-  GLubyte pixel[4];
-  int y_off;
-  int x_off;
   gboolean missing_feature = FALSE;
 
   if (rect0_state)
@@ -126,32 +100,14 @@ test_depth (TestState *state,
   if (missing_feature)
     return;
 
-  /* See what we got... */
-
-  y_off = y * QUAD_WIDTH + (QUAD_WIDTH / 2);
-  x_off = x * QUAD_WIDTH + (QUAD_WIDTH / 2);
-
-  cogl_read_pixels (x_off, y_off, 1, 1,
-                    COGL_READ_PIXELS_COLOR_BUFFER,
-                    COGL_PIXEL_FORMAT_RGBA_8888_PRE,
-                    pixel);
-
-  check_pixel (pixel, expected_result);
+  test_utils_check_pixel (x * QUAD_WIDTH + (QUAD_WIDTH / 2),
+                          y * QUAD_WIDTH + (QUAD_WIDTH / 2),
+                          expected_result);
 }
 
 static void
 paint (TestState *state)
 {
-  CoglMatrix identity;
-
-  cogl_ortho (0, state->width, /* left, right */
-              state->height, 0, /* bottom, top */
-              -1, 100 /* z near, far */);
-
-  cogl_push_matrix ();
-  cogl_matrix_init_identity (&identity);
-  cogl_set_modelview_matrix (&identity);
-
   /* Sanity check a few of the different depth test functions
    * and that depth writing can be disabled... */
 
@@ -270,8 +226,6 @@ paint (TestState *state)
                 &rect0_state, &rect1_state, NULL,
                 0x00ff00ff); /* expected */
   }
-
-  cogl_pop_matrix ();
 }
 
 void
@@ -281,8 +235,10 @@ test_cogl_depth_test (TestUtilsGTestFixture *fixture,
   TestUtilsSharedState *shared_state = data;
   TestState state;
 
-  state.width = cogl_framebuffer_get_width (shared_state->fb);
-  state.height = cogl_framebuffer_get_height (shared_state->fb);
+  cogl_ortho (0, cogl_framebuffer_get_width (shared_state->fb), /* left, right */
+              cogl_framebuffer_get_height (shared_state->fb), 0, /* bottom, top */
+              -1, 100 /* z near, far */);
+
   paint (&state);
 
   if (g_test_verbose ())
